
/*
 * Copyright (C) Yichun Zhang (agentzh)
 */


#ifndef _NGX_HTTP_LUA_WORKER_H_INCLUDED_
#define _NGX_HTTP_LUA_WORKER_H_INCLUDED_


#include "ngx_http_lua_common.h"


void ngx_http_lua_inject_worker_api(lua_State *L);


#endif /* _NGX_HTTP_LUA_WORKER_H_INCLUDED_ */
